using System;
using System.Threading;
using System.Runtime.Remoting.Contexts;

[Synchronization(SynchronizationAttribute.REQUIRED)]
class Counter : ContextBoundObject
{
	private long counter = 0;
	public void IncrementCounter()
	{
		long oldValue = counter;
		oldValue += 1;
		Thread.Sleep(0);
		counter = oldValue;
	}

	public long GetCounter()
	{
		return counter;
	}

}

class Action
{
	private Counter counter;

	public Action (int start, int increment, int end, Counter counter)
	{
		this.start = start;
		this.increment = increment;
		this.end = end;
		this.counter = counter;
	}

	private int start;
	private int increment;
	private int end;

	public void Count()
	{
		Console.WriteLine("Thread {0} enters Count", Thread.CurrentThread.GetHashCode());
		for (int i = start; i <= end; i += increment)
		{
			Console.WriteLine("Thread {0} has value {1}", Thread.CurrentThread.GetHashCode(), i);
			counter.IncrementCounter();	
		}
		Console.WriteLine("Thread {0} exits Count", Thread.CurrentThread.GetHashCode());
	}

}

class Example
{
	static void Main()
	{
		Counter counter = new Counter();

		Action a1 = new Action(1, 10, 100, counter);
		Action a2 = new Action(100, 10, 200, counter);

		ThreadStart ts1 = new ThreadStart(a1.Count);
		ThreadStart ts2 = new ThreadStart(a2.Count);

		Thread t1 = new Thread(ts1);
		Thread t2 = new Thread(ts2);

		t1.Start();
		t2.Start();

		t1.Join();
		t2.Join();

		Console.WriteLine("Number of counts: {0}", counter.GetCounter());

	}
}

